#! /bin/sh


alias echo_date='echo 【$(TZ=UTC-8 date -R +%Y年%m月%d日\ %X)】:'
export KSROOT=/jffs/softcenter
source $KSROOT/scripts/base.sh
eval `dbus export merlinclash`
SOFT_DIR=/jffs/softcenter
KP_DIR=$SOFT_DIR/merlinclash/koolproxy
lan_ipaddr=$(nvram get lan_ipaddr)
LOCK_FILE=/var/lock/koolproxy.lock
LOG_FILE=/tmp/merlinclash_log.txt

OS=$(uname -r)
#=======================================

set_lock(){
	exec 1000>"$LOCK_FILE"
	flock -x 1000
}

unset_lock(){
	flock -u 1000
	rm -rf "$LOCK_FILE"
}
urldecode(){
  echo -e "$(sed 's/+/ /g;s/%\(..\)/\\x\1/g;')"
}
rm -rf /tmp/clash_routerrules.log
rm -rf /tmp/clash_routerrules.txt
b(){
	if [ -f "/bin/base64" ]; then #HND是这个
		base=base64
		echo $base
	elif [ -f "/jffs/softcenter/bin/base64_decode" ]; then #HND有这个
		base=base64_decode
		echo $base
	elif [ -f "/jffs/softcenter/bin/base64" ]; then #网件R7K是这个
		base=base64
		echo $base
	elif [ -f "/sbin/base64" ]; then
		base=base64
		echo $base
	else
		echo_date "【错误】固件缺少base64decode文件，无法正常订阅，直接退出" >> $LOG_FILE
		echo_date "解决办法请查看MerlinClash Wiki" >> $LOG_FILE
		echo BBABBBBC >> $LOG_FILE
		exit 1
	fi
}
detect_domain() {
	domain1=$(echo $1 | grep -E "^https://|^http://")
	domain2=$(echo $1 | grep -E "\.")
	if [ -n "$domain1" ] || [ -z "$domain2" ]; then
		return 1
	else
		return 0
	fi
}
detect_ip(){	
	IPADDR=$1
	regex_v4="\b(25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9][0-9]|[1-9])\.(25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9][0-9]|[0-9])\.(25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9][0-9]|[0-9])\.(25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9][0-9]|[1-9])\b"
	regex_v6="(([0-9a-fA-F]{1,4}:){7,7}[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,7}:|([0-9a-fA-F]{1,4}:){1,6}:[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,5}(:[0-9a-fA-F]{1,4}){1,2}|([0-9a-fA-F]{1,4}:){1,4}(:[0-9a-fA-F]{1,4}){1,3}|([0-9a-fA-F]{1,4}:){1,3}(:[0-9a-fA-F]{1,4}){1,4}|([0-9a-fA-F]{1,4}:){1,2}(:[0-9a-fA-F]{1,4}){1,5}|[0-9a-fA-F]{1,4}:((:[0-9a-fA-F]{1,4}){1,6})|:((:[0-9a-fA-F]{1,4}){1,7}|:)|fe80:(:[0-9a-fA-F]{0,4}){0,4}%[0-9a-zA-Z]{1,}|::(ffff(:0{1,4}){0,1}:){0,1}((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])|([0-9a-fA-F]{1,4}:){1,4}:((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9]))"
	ckStep4=`echo $1 | egrep $regex_v4 | wc -l`
	ckStep6=`echo $1 | egrep $regex_v6 | wc -l`
	if [ $ckStep4 -eq 0 ]; then
		if [ $ckStep6 -eq 0 ]; then
			return 1
		else
			return 6
		fi
	else
		return 4
	fi
}
decode_url_link(){
	local link=$1
	local len=$(echo $link | wc -L)
	local mod4=$(($len%4))
	b64=$(b)
	echo_date "b64=$b64" >> LOG_FILE
	if [ "$mod4" -gt "0" ]; then
		local var="===="
		local newlink=${link}${var:$mod4}
		echo -n "$newlink" | sed 's/-/+/g; s/_/\//g' | $b64 -d 2>/dev/null
	else
		echo -n "$link" | sed 's/-/+/g; s/_/\//g' | $b64 -d 2>/dev/null
	fi
}
get(){
	a=$(echo $(dbus get $1))
	a=$(echo $(dbus get $1))
	echo $a
}
yamlname=$(get merlinclash_yamlsel)
yamlpath=/jffs/softcenter/merlinclash/yaml_use/$yamlname.yaml
dnslistenport=$(cat $yamlpath | awk -F: '/listen/{print $3}' | xargs echo -n)
action=$(get merlinclash_action)
restart_dnsmasq() {
    # Restart dnsmasq
	echo_date "重启 dnsmasq..." >> $LOG_FILE
    service restart_dnsmasq >/dev/null 2>&1
}
saverule(){
	rm -rf /jffs/softcenter/merlinclash/conf/clash_router_ipset.conf
	echo_date "开始生成路由经代理自定义规则文件" > $LOG_FILE
	count=$(get merlinclash_router_rule_count)
	if [ -n "$count" ];then
		i=0
		while [ "$i" -lt "$count" ]
		do
			txt=$(get merlinclash_router_rule_$i)
			#开始拼接文件值，然后进行base64解码，写回文件
			content=${content}${txt}
			let i=i+1
		done
		echo $(decode_url_link $content) > /tmp/routerrule.txt
		if [ -f /tmp/routerrule.txt ]; then
			echo_date "中间文件已经创建" >> $LOG_FILE
			echo_date "生成新文件" >> $LOG_FILE
			cat /tmp/routerrule.txt | urldecode > /jffs/softcenter/merlinclash/yaml_basic/routerrules.yaml
			rm -rf /tmp/routerrule.txt
		fi
		#dbus remove jdqd_jd_script_content_custom
		customs=`dbus list merlinclash_router_rule_ | cut -d "=" -f 1`
		for custom in $customs
		do
			dbus remove $custom
		done
		if [ -f "/jffs/softcenter/merlinclash/yaml_basic/routerrules.yaml" ]; then
  			ln -sf /jffs/softcenter/merlinclash/yaml_basic/routerrules.yaml /tmp/clash_routerrules.txt 
			echo_date "【完成】/jffs/softcenter/merlinclash/yaml_basic/routerrules.yaml文件创建成功！" >> $LOG_FILE
		fi
	fi
	
}

apply_rule(){
	#echo "apply_rule" >> /tmp/1.txt
	ipset -F router >/dev/null 2>&1
	jdqd=$(get jdqd_jd_enable)
	if [ "$jdqd" == "1" ]; then
		ipset add router 149.154.160.0/20 #检测到京东签到，写入TG IP段，使得TGBOT能用
	fi
	a=$(ipset list router)
	sed -i '/^ *$/d' /jffs/softcenter/merlinclash/yaml_basic/routerrules.yaml
	if [ -s "/jffs/softcenter/merlinclash/yaml_basic/routerrules.yaml" ] && [ -n "$a" ]; then
		if [ "$1" != "apply" ]; then
			echo_date "创建路由经代理访问ipset规则集" >> $LOG_FILE
		fi
		rm -rf /tmp/ipset_router.list
		cp -rf /jffs/softcenter/merlinclash/yaml_basic/routerrules.yaml /tmp/ipset_router.list 2>/dev/null
		rm -rf /tmp/clash_router_tmp.txt
		rm -rf /tmp/clash_router_tmp2.txt
		lines=$(cat /tmp/ipset_router.list | awk '{print $0}')
		#sleep 2s
		#echo "sleep 2s"
		#echo lines=$lines
		for line in $lines
		do
			#先检测是否为IP格式
			detect_ip ${line}
			a=$?
			if [ "$a" == "4" ]; then
				echo_date "${line}为合法IPv4格式，进行处理" >> $LOG_FILE
				#echo ${line} >> /tmp/clash_ipsetproxy.txt
				ipset -! add router ${line} >/dev/null 2>&1
			elif [ "$a" == "6" ]; then
				echo_date "${line}为合法IPv6格式，目前未支持IPV6地址，略过" >> $LOG_FILE 
				continue
			else
				echo_date "${line}不为IP格式，检查是否为域名地址" >> $LOG_FILE 
				detect_domain ${line}
				if [ "$?" == "0" ]; then
					echo_date "${line}为合法域名格式，进行处理" >> $LOG_FILE 
					nslookup "${line}" 127.0.0.1:${dnslistenport} > /tmp/clash_router_tmp.txt
					#sleep 3s
					#echo "sleep 3s"
					cat /tmp/clash_router_tmp.txt | grep -n "^Address" | awk -F " " '{print $3}' | grep -v "127.0.0.1" > /tmp/clash_router_tmp2.txt #域名解析结果可能含有V4跟V6地址。下一步再进行筛选
					#sleep 4s
					#echo "sleep 4s"
					lines=$(cat /tmp/clash_router_tmp2.txt | awk '{print $0}')
					#echo lines2=$lines
					for line2 in $lines
					do
						detect_ip ${line2}
						b=$?
						if [ "$b" == "4" ]; then
							echo_date "${line2}为合法IPV4格式，进行处理" >> $LOG_FILE 
							ipset -A router ${line2}
						elif [ "$b" == "6" ]; then
							#echo "为合法IPV6格式，进行处理" 
							continue
						fi
					done
					echo "$line" | sed "s/^/ipset=&\/./g" | sed "s/$/\/router/g" >> /jffs/softcenter/merlinclash/conf/clash_router_ipset.conf
				else
					echo_date "格式有误，略过" >> $LOG_FILE  
				fi
			fi
		done
	else
		rm -rf /jffs/softcenter/merlinclash/conf/clash_router_ipset.conf
	fi
	rm -rf /jffs/configs/dnsmasq.d/clash_router_ipset.conf >/dev/null 2>&1
	if [ -f "/jffs/softcenter/merlinclash/conf/clash_router_ipset.conf" ]; then
		ln -sf /jffs/softcenter/merlinclash/conf/clash_router_ipset.conf /jffs/configs/dnsmasq.d/clash_router_ipset.conf >/dev/null 2>&1
		echo_date "【完成】/jffs/softcenter/merlinclash/conf/clash_router_ipset.conf文件创建成功！" >> $LOG_FILE  
	fi
	if [ "$2" != "rr" ]; then
		restart_dnsmasq
	fi
}
case $1 in
apply)
	apply_rule apply "rr"
	;;
esac
case $action in
45)
	echo_date "生成路由经代理自定义规则文件" >> /tmp/clash_routerrules.log
	echo_date "生成路由经代理自定义规则文件" >> $LOG_FILE
	saverule
	apply_rule
	echo BBABBBBC >> $LOG_FILE
	;;
esac